import random
from django.db import models
from colors.models import Color
from customers.models import Customer
from gauge.models import Gauge
from products.models import Product
from uom.models import UnitOfMeasure
from account.models import Account
from django.db import models, IntegrityError



DELIVERY_MODE_CHOICES = [
        ('Free Delivery', 'Free Delivery'),
        ('Self Collection', 'Self Collection'),
    ]
PAYMENT_TERMS_CHOICES = [
        ('Cash with Order', 'Cash with Order'),
        ('Lipa Mdogo Mdogo', 'Lipa Mdogo Mdogo'),
        ('Partial Payment', 'Partial Payment'),
    ]
QUOTATION_STATUS = [
        ('APPROVED', 'APPROVED'),
        ('REJECTED', 'REJECTED'),
        ('PENDING', 'PENDING'), 
        ('In Production', 'In Production'),
        ('PAID', 'PAID'), 
         ('PARTIAL PAID', 'PARTIAL PAID'),
        ('Produced', 'Produced'),
        ('Shipped', 'Shipped'),
        ('Dispatched', 'Dispatched'),
        ('Recieved', 'Recieved'),
    ]
LEAD_SOURCE_CHOICES = [
        ('Walk-ins', 'Walk-ins'),
        ('Facebook', 'Facebook'),
        ('Website', 'Website'),
        ('Referral', 'Referral'),
        ('Fundi', 'Fundi'),
        ('Telephone', 'Telephone'), 
    ]
PAYMENT_STATUS_CHOICE = [
    
]

def generate_quotation_code(): 
    random_number = str(random.randint(0, 999999)).zfill(6) 
    product_code = f'QID{random_number}' 
    return product_code

class Quotation(models.Model):
    qid = models.CharField(max_length=100, unique=True, default=generate_quotation_code)
    qr_code = models.ImageField(upload_to='qr_codes/', blank=True, null=True)
    customer_info = models.ForeignKey(Customer, on_delete=models.CASCADE)
    seller = models.ForeignKey(Account, on_delete=models.CASCADE, null=True, blank=True)
    total_amount = models.DecimalField(max_digits=10, decimal_places=2, null=True, blank=True)
    amount_paid = models.DecimalField(max_digits=10, default=0, decimal_places=2, null=True, blank=True)
    status = models.CharField(max_length=20, choices=QUOTATION_STATUS, default="PENDING", null=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.qid:
            while True:
                new_qid = generate_quotation_code()
                if not Quotation.objects.filter(qid=new_qid).exists():
                    self.qid = new_qid
                    break
        super().save(*args, **kwargs)

DISCOUNT_STATUS_CHOICES = [
    ('Requested', 'Requested'),
    ('Rejected', 'Rejected'),
    ('Approved', 'Approved'),
    ('Not Applied', 'Not Applied'),
]

class Inquiry(models.Model):
    quotation = models.ForeignKey(Quotation, on_delete=models.CASCADE)
    product = models.ForeignKey(Product, on_delete=models.CASCADE, blank=True, null=True)
    seller = models.ForeignKey(Account, on_delete=models.CASCADE, null=True, blank=True)
    updated_by = models.ForeignKey(Account, on_delete=models.CASCADE, related_name='updated_inquiries', null=True, blank=True)
    gauge = models.ForeignKey(Gauge, on_delete=models.CASCADE, null=True, blank=True)
    color = models.ForeignKey(Color, on_delete=models.CASCADE, null=True, blank=True)
    size = models.DecimalField(max_digits=10, decimal_places=2, blank=True, null=True)
    uom = models.ForeignKey(UnitOfMeasure, on_delete=models.CASCADE, blank=True, null=True)
    quantity = models.PositiveIntegerField(blank=True, null=True)
    unit_price = models.DecimalField(max_digits=10, decimal_places=2, blank=True, null=True)
    amount = models.DecimalField(max_digits=10, decimal_places=2, blank=True, null=True)
    discount_amount = models.DecimalField(max_digits=10, decimal_places=2, null=True, blank=True)
    discount_status = models.CharField(max_length=50, choices=DISCOUNT_STATUS_CHOICES, default='Not Applied')
    discount_validator = models.ForeignKey(Account, on_delete=models.CASCADE, related_name="discount_validator", null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True, null=True, blank=True)
    produced_quantity = models.DecimalField(max_digits=10, decimal_places=2, default=0, blank=True, null=True)
    status = models.CharField(max_length=50, default='Pending', blank=True, null=True)
    is_coil_requested = models.BooleanField(default=False)
    meters = models.DecimalField(max_digits=10, decimal_places=2, default=0)  
    total_meters = models.DecimalField(max_digits=12, decimal_places=2, default=0.0) 
    items_made = models.PositiveIntegerField(default=0)
    PRODUCTION_STATUS_CHOICES = [
        ('Paid', 'Paid'),
        ('In Production', 'In Production'),
        ('Produced', 'Produced'),
        ('Shipped', 'Shipped'), 
    ]

    production_status = models.CharField(
        max_length=20,
        choices=PRODUCTION_STATUS_CHOICES,
        default='Paid',
    )

    def __str__(self):
        return self.quotation.qid

    def save(self, *args, **kwargs):
        """
        Calculates and sets the `meters` field automatically whenever the model is saved.
        """
        if self.size and self.quantity:
            self.meters = self.size * self.quantity
        super().save(*args, **kwargs)

    @property
    def total_meters_produced(self):
        """
        Calculates the total meters produced based on produced_quantity and potentially adjusts total_meters.

        This approach is flexible and allows for manual adjustments of total_meters if needed.
        """
        if self.produced_quantity:
            # Consider potential size variations during production:
            produced_meters = self.produced_quantity * self.size  # Adjust if size can vary
            # Potentially update total_meters based on production:
            # self.total_meters = max(self.total_meters, produced_meters)  # Uncomment for automatic adjustment
            return produced_meters
        else:
            return 0

    def __str__(self):
        return f"Inquiry: {self.quotation.qid} - {self.product}"

class CustomerQuoteInfo(models.Model):
    customer = models.ForeignKey(Customer, on_delete=models.CASCADE)
    quotation = models.ForeignKey(Quotation, on_delete=models.CASCADE)
    seller = models.ForeignKey(Account, on_delete=models.CASCADE, null=True, blank=True)
    delivery_location = models.CharField(max_length=20, blank=True, null=True)
    delivery_mode = models.CharField(max_length=20, choices=DELIVERY_MODE_CHOICES, default="Free Delivery", blank=True, null=True)
    payment_terms = models.CharField(max_length=20, choices=PAYMENT_TERMS_CHOICES, default="Cash with Order", blank=True, null=True)
    lead_source = models.CharField(max_length=50, choices=LEAD_SOURCE_CHOICES, default="Walk-ins", blank=True, null=True)